package partisan.weforge.xyz.pulse

import android.content.Context
import androidx.core.content.edit
import androidx.preference.PreferenceManager

class Preferences(ctx: Context) {
    companion object {
        private const val ENABLED = "enabled"
        private const val REDIRECTION_DELAY = "redirection_delay"
        private const val POPUP_POSITION = "popup_position_y"

        private const val DEFAULT_REDIRECTION_DELAY = 2000L
        private const val DEFAULT_POPUP_POSITION = 333

        // migration
        private const val SERVICE_ENABLED = "service_enabled"
    }

    private val prefs = PreferenceManager.getDefaultSharedPreferences(ctx)

    var isEnabled: Boolean
        get() = prefs.getBoolean(ENABLED, prefs.getBoolean(SERVICE_ENABLED, false))
        set(value) = prefs.edit { putBoolean(ENABLED, value) }

    var redirectionDelay: Long
        get() = prefs.getLong(REDIRECTION_DELAY, DEFAULT_REDIRECTION_DELAY)
        set(value) = prefs.edit { putLong(REDIRECTION_DELAY, value) }

    var popupPosition: Int
        get() = prefs.getInt(POPUP_POSITION, DEFAULT_POPUP_POSITION)
        set(value) = prefs.edit { putInt(POPUP_POSITION, value) }

    private fun makeKeyEnabled(mimetype: String) = "enabled_$mimetype"
    private fun makeKeyPriority(mimetype: String) = "priority_$mimetype"

    /** Whether this service is enabled */
    fun isServiceEnabled(mimetype: String): Boolean {
        return prefs.getBoolean(makeKeyEnabled(mimetype), true)
    }

    /** Current priority for this service (lower = higher priority) */
    fun getServicePriority(mimetype: String): Int {
        return prefs.getInt(makeKeyPriority(mimetype), Int.MAX_VALUE)
    }

    /** Enable or disable individual service */
    fun setServiceEnabled(mimetype: String, enabled: Boolean) {
        prefs.edit().putBoolean(makeKeyEnabled(mimetype), enabled).apply()
    }

    /** Change priority for an individual service */
    fun setServicePriority(mimetype: String, priority: Int) {
        prefs.edit().putInt(makeKeyPriority(mimetype), priority).apply()
    }
}
